/********************************************************************************\
 * Eagle LegaliZer: Makes any "pirate-marked" Eagle File become an "Original".	*
 * By ViKT0RY - 2005-2016							*
 * http://www.vikt0ry.com							*
\********************************************************************************/ 

#include <windows.h>

#define elemof(x) (sizeof(x)/sizeof(*(x)))

#pragma comment(linker,"/OPT:NOWIN98 /MERGE:.rdata=.text /IGNORE:4078")

unsigned long startvalue;	// this data is moved from check to patch

char outbuffer[20000];
char *outaux=outbuffer;

#pragma pack(1)
struct rec_eagle{
 char rectype;	// 0x10 = Eagle Header
 char inuse;	// 0x00 = untouched, 0x80 = touched
 short number_of_checkpoints;
 long number_of_records;
 short version1,version2;
 char checksum2,checksum;
 char unused[10];
};
struct block11b{
 long dword1,dword2;
 short word;
 char byte;
};
struct rec_check{
 char rectype;	// 0x11 = Copy protection check record
 char inuse;
 block11b block[2];
};
#pragma pack()

static char make_checksum(const char *p) {
 char value=0;
 for (int i=24; i<3*24; i+=24) {	// Two 0x11 records
  long checksum=p[i];			// initialize with 0x11 here
  for (int j=2; j<24; j++) {
   checksum<<=1;
   if (checksum&0x80000000) checksum^=0x80000001;
   checksum+=p[i+j];
  }
  value+=char(checksum>>24^checksum>>16^checksum>>8^checksum);
 }
 value^=p[4];
 return value;
}

static bool checkdate(long value) {
 int magic=value>>16&0xFF;
 if (magic>53) return false;
 magic=value>>24&0xFF;		// year
 if (magic<95) return false;	// 1995
 if (magic>114) return 0;	// 2014
 if (!(value&0xFFFF)) return false;
 magic-=94;
 magic*=3000;		/* magic should be 12000 */
 if ((value&0xFFFF)>=magic) return false;
 return true;
}


static void crypt(unsigned long *first, unsigned long second) {
 *first=((((((~*first) >> 8) ^ second) >> 8) ^ second) >> 8) ^ second;
}

static void make_first_xor(char p[9]) {
 char xorvalue=(char)0xFC;
 for (int j=0;j<9;j++) {
  p[j]^=xorvalue;
  xorvalue+=0x42;
 }
}

static void get_magic(unsigned long &magic, unsigned char &magic_checksum) {
 unsigned long ret=0;
 magic=startvalue;
 while (!ret) {
  magic+=1;
  if (magic<0x9d800000) magic=0x9d800000;
  unsigned long last_value1=magic;
  char xorvalue=(char)0xFC;
  char *p=(char *)&last_value1;
  for (int j=0;j<4;j++) {
   p[j]^=xorvalue;
   xorvalue+=0x42;
  }
  p=(char*)&last_value1;
  unsigned char a=(unsigned char)p[3];	// year-1900
  unsigned char b=(unsigned char)p[2];
  if (a<95 || b>52 || a>114) {
   ret=0;
   continue;
  }
  unsigned short value1=*(unsigned short *)(&last_value1);
  unsigned short value2=2992 * (a-94);
  if (value1>value2) {
   ret=0;
   continue;
  }
  unsigned long last_value2=last_value1;
  crypt(&last_value2,last_value2);
  magic_checksum=(char)last_value2^xorvalue;
  ret=magic^0x68637324;
  switch (ret) {
   case 0x0A7A6DA0:
   case 0x096962F4:
   case 0x0E533B97:
   case 0x044F621B:
   case 0x09447572:
   case 0x376279D8: ret=0; continue;
  }
  ret++;
 }
}

static int fixmem(char*lpBaseAddress) {
 unsigned long magic[2]={0};
 unsigned char magic_checksum[2]={0};

 get_magic(magic[0],magic_checksum[0]);
 get_magic(magic[1],magic_checksum[1]);

 rec_eagle*eaglehdr=(rec_eagle*)lpBaseAddress;

 for (int offset=0,i=eaglehdr->number_of_checkpoints;offset+=24,i>=0;i--) {
  rec_check*eaglechk=(rec_check*)(lpBaseAddress+offset);
  if (eaglechk->rectype!=0x11) continue;
  for(int j=0;j<2;j++) {
   block11b*p=&eaglechk->block[j];
   p->dword1=magic[j];
   p->dword2=magic_checksum[j] | 0x90000;
   p->word=0;
   p->byte=0;
  }
 }
 eaglehdr->checksum=make_checksum(lpBaseAddress);
 eaglehdr->checksum2=0;
 return 1;
}

/* This function fixes the file so that Legal Eagle can open it */
static int fixfile(const char*filename){
 int result=-1;
 HANDLE hFile=CreateFile(filename,GENERIC_READ|GENERIC_WRITE,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
 if (hFile!=INVALID_HANDLE_VALUE) {
  HANDLE hMapped=CreateFileMapping(hFile,NULL,PAGE_READWRITE,0,0,NULL);
  if (hMapped) {
   char*lpBaseAddress=(char*)MapViewOfFile(hMapped,FILE_MAP_ALL_ACCESS,0,0,0);
   if (lpBaseAddress) {
    result=fixmem(lpBaseAddress);
    UnmapViewOfFile(lpBaseAddress);
   }
   CloseHandle(hMapped);
  }else outaux+=wsprintf(outaux,"Error creating file mapping\n");
  CloseHandle(hFile);
 }else outaux+=wsprintf(outaux,"Couldn't open '%s' for writing!\n", filename);
 return result;
}

static int checkmem(const char*lpBaseAddress) {
 bool okay=false;
 const rec_eagle*eaglehdr=(rec_eagle*)lpBaseAddress;
 if(eaglehdr->rectype!=0x10) {
  outaux+=wsprintf(outaux,"File has improper format!\n");
  return 2;
 }
 short version=eaglehdr->version1;
 version=version>>8|version<<8;
 for (int offset=0,i=eaglehdr->number_of_checkpoints;offset+=24,i>=0;i--) {
  const rec_check*eaglechk=(const rec_check*)(lpBaseAddress+offset);
  if (eaglechk->rectype!=0x11) continue;
  for (int counter=0; counter<2; counter++) {
   block11b buffer=eaglechk->block[counter];	// copy 11 bytes
   startvalue=buffer.dword1;		// save something required later
   make_first_xor((char*)&buffer);	// scramble 9 of 11 bytes
   unsigned long last_value=buffer.dword1;
   if (buffer.dword1) {
    crypt(&last_value,buffer.dword1);
    char ret1=(char)last_value;
    crypt(&last_value,buffer.dword1);
    char ret2=(char)last_value;
    char tempxor=(char)buffer.dword2;
    if (ret1!=tempxor) return 0;
    long temp=buffer.dword1^0x68637324;
    if ((buffer.dword2 & 0xF0000)!=0x10000) return 0;
    if (!checkdate(buffer.dword1)) return 0;
    switch (temp) {
     case 0x096962F4: if (okay || version<0x40A) goto def;
     case 0x0A7A6DA0:
     case 0x0E533B97:
     case 0x044F621B:
     case 0x09447572:
     case 0x376279D8: return 0;
     default: def: okay=true;
    }
   }
  }
 }
 if (!okay) return 0;
 if (make_checksum(lpBaseAddress)!=eaglehdr->checksum) return 0;
 switch (eaglehdr->checksum2) {
  case 0x80:
  case 0x40: return 0;
 }
 return 1;
}

/* This function makes the same comprobations as original Eagle source
   Return values:
	-1 = could not open file, message was generated
	 0 = file is illegal
	 1 = file is legal
	 2 = Not an eagle file, message was generated
 */
static int checkfile(const char*filename){
 int result=-1;
 HANDLE hFile=CreateFile(filename,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
 if(hFile!=INVALID_HANDLE_VALUE) {
  HANDLE hMapped=CreateFileMapping(hFile,NULL,PAGE_READONLY,0,0,NULL);
  if (hMapped) {
   const char*lpBaseAddress=(const char*)MapViewOfFile(hMapped,FILE_MAP_READ,0,0,0);
   if (lpBaseAddress) {
    result=checkmem(lpBaseAddress);
    UnmapViewOfFile(lpBaseAddress);
   }
   CloseHandle(hMapped);
  }else outaux+=wsprintf(outaux,"Error creating file mapping\n");
  CloseHandle(hFile);
 }else outaux+=wsprintf(outaux,"Couldn't open '%s' for reading!\n",filename);
 return result;
}

static void ParseCmdLine(char *cmd, int *argc, char *argv[256]){
 int i=0;
 char *p=cmd;
 int parametro=0;

 while((i<256) && (lstrlen(p)>0)) {
  if(*p!=' '){
   if(*p=='"') {
    if(parametro==0){
     parametro=1;
     argv[i]=p + 1;
     i++;
    } else {
     parametro=0;
     *p=0;
    }
   } else {
    if(parametro==0){
     parametro=2;
     argv[i]=p;
     i++;
    }
   }
  } else {
   if(parametro==2){
    parametro=0;
    *p=0;
   }
  }
  p++;
 }
 *argc=i;
}

static void AddContextMenu(char *extension,char *data){
 HKEY hkey;
 if(RegOpenKey(HKEY_CLASSES_ROOT,extension,&hkey)==ERROR_SUCCESS) {
  char temp[512];
  DWORD dwsize=sizeof temp;
  HKEY hkey2;
  RegQueryValueEx(hkey,0,0,0,(BYTE*)temp,&dwsize);
  if(!lstrlen(temp) || !lstrcmpi(temp,"Eagle_LegaliZer")) {
   RegSetValueEx(hkey,0,0,REG_SZ,(BYTE*)"Eagle_LegaliZer",lstrlen("Eagle_LegaliZer")+1);
   RegCloseKey(hkey);
   if(RegCreateKey(HKEY_CLASSES_ROOT,temp,&hkey)!=ERROR_SUCCESS) {
    RegOpenKey(HKEY_CLASSES_ROOT,temp,&hkey);
   }
  }else{
   RegCloseKey(hkey);
   RegOpenKey(HKEY_CLASSES_ROOT,temp,&hkey);
  }
  if(RegCreateKey(hkey,"shell\\Fix with Eagle LegaliZer",&hkey2)==ERROR_SUCCESS) {
   RegCloseKey(hkey);
   if(RegCreateKey(hkey2,"command",&hkey)==ERROR_SUCCESS) {
    RegSetValueEx(hkey,0,0,REG_SZ,(BYTE*)data,lstrlen(data)+1);
    RegCloseKey(hkey);
    RegCloseKey(hkey2);
   }
  }
 }
}

static void RemoveContextMenu(char *extension){
 HKEY hkey;
 if (RegOpenKey(HKEY_CLASSES_ROOT,extension,&hkey)==ERROR_SUCCESS) {
  char v[512];
  long size=sizeof v;
  RegQueryValue(hkey,NULL,v,&size);
  RegCloseKey(hkey);
  if(RegOpenKeyEx(HKEY_CLASSES_ROOT,v,0,KEY_ALL_ACCESS,&hkey)==ERROR_SUCCESS) {
   RegDeleteKey(hkey,"shell\\Fix with Eagle LegaliZer\\command");
   RegDeleteKey(hkey,"shell\\Fix with Eagle LegaliZer");
   RegCloseKey(hkey);
  }
 }
}

static void MakeInstall(void){
 char path[256];
 GetWindowsDirectory(path,sizeof path);
 lstrcat(path,"\\Eagle_LegaliZer.exe \"%1\"");
 AddContextMenu(".brd",path);
 AddContextMenu(".sch",path);
 AddContextMenu(".lbr",path);
}


static void RemoveInstall(void){
 RemoveContextMenu(".brd");
 RemoveContextMenu(".sch");
 RemoveContextMenu(".lbr");
}

int _stdcall WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance, LPSTR lpCmdLine,int nCmdShow) {

 int ret=0;
 int i=1;
 bool force=false;
 bool checkonly=false;
 char*arg;
 int argc;
 char *argv[256];

 ParseCmdLine(lpCmdLine, &argc,argv);

 arg=argv[i];

 if (argc>=2) switch (arg[0]) {
  case '-':
  case '/': switch (arg[1]|0x20) {
   case 'i': MakeInstall(); return 0;
   case 'd': RemoveInstall(); return 0;
   case 'c': checkonly=true; i++; break;
   case 'f': force=true; i++; break;
  }break;
 }

 outaux+=wsprintf(outaux,"\n");
 outaux+=wsprintf(outaux,"\t\t\t\t  \n");
 outaux+=wsprintf(outaux," Eagle LegaliZer v1.2 by ViKT0RY\t  \n");
 outaux+=wsprintf(outaux,"\t\t\t\t  \n");
 outaux+=wsprintf(outaux," http://www.vikt0ry.com\t\t  \n");
 outaux+=wsprintf(outaux,"\t\t\t\t  \n");
 outaux+=wsprintf(outaux,"\n\n");

 if(i>=argc) {
  outaux+=wsprintf(outaux,"You must specify a file to patch!\n\nUsage: Eagle_LegaliZer.exe [-f] File.[sch]|[brd]|[lbr] ...\n");
  outaux+=wsprintf(outaux,"\nThis means that you can legalize any schema, board or library, use the parameter -f to force repair.");
  MessageBox(0,outbuffer,"Notice",MB_ICONEXCLAMATION);
  return 0;
 }
 
 for(;i<argc;i++) {
  arg=argv[i];
  ret=checkfile(arg);
 
  if(ret==1) {
   if(!force) {
    outaux+=wsprintf(outaux,"The file '%s' seems to be legal.\n", arg);
    continue;
   }
  } else if(ret==2 || ret==-1) {
   continue;
  }
 
  if (checkonly) {
   outaux+=wsprintf(outaux,"File '%s' is illegal.\n",arg);
  }else{
   ret=fixfile(arg);
   if(ret==0) {
    outaux+=wsprintf(outaux,"Couldn't fix the file '%s'!\n",arg);
   }else{
    outaux+=wsprintf(outaux,"File '%s' fixed successfully.\n", arg);
   }
  }
 }
 MessageBox(0,outbuffer,"Notice",MB_ICONINFORMATION);
 return 0;
}

EXTERN_C void CALLBACK WinMainCRTStartup() {
 ExitProcess(WinMain(0,0,GetCommandLine(),0));
}
